/*
 * Turbo Vision reportflag
 * (C) 2008, Hans-Christian Esperer
 * Released under the new (3-clause) BSD license.
 */


#define Uses_TApplication
#define Uses_TButton
#define Uses_TMemo
#define Uses_TDeskTop
#define Uses_TDialog
#define Uses_MsgBox
#define Uses_TKeys
#define Uses_TMenuBar
#define Uses_TMenuItem
#define Uses_TInputLine
#define Uses_TParamText
#define Uses_TStatusDef
#define Uses_TLabel
#define Uses_TStatusItem
#define Uses_TScrollBar
#define Uses_TIndicator
#define Uses_TStatusLine
#define Uses_TSubMenu
#define Uses_TWindow

#include <stdlib.h>
#include <fcntl.h>
#include <sys/wait.h>
#include <tvision/tv.h>
#include <libutil.h>

#include "djb.h"

extern char TEXT_radv[];

enum
{
	cmAbout	= 101,
	cmReportFlag,
	cmReportAdvisory,
	cmSetIP,
	cmSetTeamNumber,
	cmAFR,
	cmRADV,
	cmStoreFlag,
	cmRetrieveFlag
};

class TMyApp: public TApplication
{
	TCommandSet windowCommands;
public:
	TMyApp();
	static TMenuBar* initMenuBar(TRect r);
	static TStatusLine* initStatusLine(TRect r);
	void aboutBox();
	void createLifeWindow();
	TCommandSet getCommands();
	void handleEvent(TEvent& event);
	void idle();
	void doReportFlag();
	void doReportAdv();
	int doSetIP();
	int doSetTeamNumber();
	int doStoreFlag();
	int doRetrieveFlag();
	int doSetHostIP();
	int testScorebotReachability();
	void howtoRADV(int always = 0);
};

		
TMyApp::TMyApp(): TProgInit(&TMyApp::initStatusLine, &TMyApp::initMenuBar,
	&TMyApp::initDeskTop), windowCommands(getCommands())
{
}

void TMyApp::aboutBox()
{
	TDialog *box = new TDialog(TRect(0, 0, 52, 14), "About");
	box->insert(new TStaticText(TRect(1, 2, 1+50, 2+8),
		"\003HC's reportflag\n"
		"\003A flag reporting utility\n\n"
		"\003Written by HC -- released under the new BSD license\n\n"
		"\003Uses Sergio Sigala's *excellent* Turbo Vision port http://www.sigala.it/sergio/tvision/"));
	box->insert(new TButton(TRect(20, 11, 20+10, 13), "~O~K", cmOK,
		bfDefault));
	box->options |= ofCentered;
	executeDialog(box);
}

TCommandSet TMyApp::getCommands()
{
	TCommandSet wc;

	wc += cmCascade;		//add window commands
	wc += cmTile;
	return wc;
}

static int
reportFlag(char* flag, char* buf, int l)
{
	FILE* f;
	char cmd[512];
	char teamid[4];
	char botip[128];
	char* nc;
	int status;
	char* tmp;

    l--;

	nc = getenv("NETCAT");
	if (nc == NULL) {
		nc = "/usr/bin/nc";
	}

	if (djbGetSetting(".ctf_scoringbot", botip, 127) < 1) {
		return 1;	
	}

	if (djbGetSetting(".ctf_teamid", teamid, 4) < 1) {
		return 1;
	}

	snprintf(cmd, 512, "%s %s 8080", nc, botip);

	f = popen(cmd, "r+");
	if (f == NULL) {
		perror("popen");
		exit(-1);
	}
	flag -= 1;
	do {
		flag += 1;
		tmp = strchr(flag, '\n');
		if (tmp != NULL) {
			*tmp = 0;
		}
		fprintf(f, "reportflag(%s, \"%s\")\n", teamid, flag);
	} while(flag = tmp);
	fputs("quit()\n", f);
	fflush(f);
	wait(&status);
	l = fread(buf, 1, l, f);
	pclose(f);

	if (l >= 0) {
		buf[l] = 0;
	} else {
		buf[0] = 0;
		return 1;
	}

	return 0;
}


static int
sendadv(char* adv)
{	
	FILE* f;
	char* nc;
	char botip[128];
	char teamid[10];
	char cmd[512];
	char buf[1024];
	int status;
	int l = 1024;

	nc = getenv("NETCAT");
	if (nc == NULL) {
		nc = "/usr/bin/nc";
	}

    l--;

	if (djbGetSetting(".ctf_scoringbot", botip, 128) < 1) {
		return 1;	
	}
	if (strlen(botip) < 1) {
		return 1;
	}

	if (djbGetSetting(".ctf_teamid", teamid, 3) < 1) {
		return 1;
	}
	if (strlen(teamid) < 1) {
		return 1;
	}


	snprintf(cmd, 512, "%s %s 8080", nc, botip);


	f = popen(cmd, "r+");
	if (f == NULL) {
		perror("popen");
		exit(-1);
	}
	fprintf(f, "reportadvisory(%s, readtext())\n", teamid);
	fprintf(f, "%s\n.\n", adv);
	fprintf(f, "quit()\n");
	fflush(f);
	wait(&status);
	l = fread(buf, 1, l, f);
	pclose(f);

	if (l >= 0) {
		buf[l] = 0;
	} else {
		buf[0] = 0;
		return 1;
	}

	if (strstr(buf, "Your advisory has been reported") == NULL) {
		return 1;
	}

	return 0;
}

/* "New advisory by     :\nAffected service(s) :"
			"\nSeverity [lmh]      :\n\n\n
*/

const char advtemplate[] = "===== Problem =====\n\n="
			"==== Impact =====\n\n===== Fix =====\n\n";

void TMyApp::doReportAdv()
{
	char* resadv;

	TDialog* box = new TDialog(TRect(0, 0, 80, 24), "Report advisory");
	TInputLine* txtBy = new TInputLine(TRect(
			25, 1, 17+25, 2), 20);
	box->insert(new TLabel(TRect(1, 1, 1+23, 2), "New ~a~dvsisory by:",
			txtBy));
	TInputLine* txtSev = new TInputLine(TRect(
			67, 1, 67+10, 2), 10);
	box->insert(new TLabel(TRect(17+25+2, 1, 65, 2), "S~e~verity:",
			txtSev));
	TInputLine* txtSrv = new TInputLine(TRect(
			25, 2, 25+25, 3), 25);
	box->insert(new TLabel(TRect(1, 2, 23, 3), "Affected "
			"~s~ervice(s):", txtSrv));
	TScrollBar* vs = new TScrollBar(TRect(1+77, 4, 1+77+1, 4+16));
	TScrollBar* hs = new TScrollBar(TRect(1, 3+16, 1+77, 3+16+1));
	TMemo* editor = new TMemo(TRect(1, 4, 1+77, 4+16), hs, vs,
			NULL, 16383);
	box->insert(new TLabel(TRect(1, 3, 1+52, 4), "A~d~visory:",
			editor));

	editor->insertText(advtemplate, sizeof(advtemplate) - 1, False);
	box->insert(vs);
	box->insert(hs);

	box->insert(txtBy);
	box->insert(txtSev);
	box->insert(txtSrv);
	box->insert(editor);

	box->insert(new TButton(TRect(60, 21, 60+10, 23), "~S~end", cmOK, bfDefault));
	box->insert(new TButton(TRect(2, 21, 2+10, 23), "~C~ancel", cmCancel, 0));

	txtBy->select();

	box->options |= ofCentered;

again:
	if (execView(box) != cmCancel) {
		resadv = new char[20480];
		editor->buffer[editor->curPtr] = 0;
		snprintf(resadv, 20480, "New advisory by : %s\nAffected service(s)"
				": %s\nSeverity [lmh] : %s\n\n\n%s%s\n",
				txtBy->data, txtSrv->data, txtSev->data, editor->buffer,
				editor->buffer + (editor->curPtr + editor->gapLen));

		suspend();
		printf("%s\n\nSENDING...\n", resadv);
		fflush(stdout);
		if (sendadv(resadv)) {
			printf("Advisory was *NOT* reported successfully!\n"
					"check your team ID and the bot IP!\n");
			printf("Press ENTER to continue\n");
			fflush(stdout);
			getchar();
			resume();
			delete resadv;
			goto again;
		} else {
			resume();
			messageBox("Advisory reported successfully.", mfOKButton |
					mfInformation);
		}
		delete resadv;
	}

	delete box;
}

void TMyApp::doReportFlag()
{
	char* buf, *tmp;
	char* msg;
	int n = -1;

	TDialog* box = new TDialog(TRect(0, 0, 70, 20), "Report flag");
	box->insert(new TButton(TRect(55, 16, 55+12, 16+2), "~R~eport", cmOK, bfDefault));
	box->insert(new TButton(TRect(2, 16, 2+12, 16+2), "~C~ancel", cmCancel, 0));
	box->insert(new TStaticText(TRect(2, 2, 2+65, 2+5),
			"Enter or paste the flags here. One flag per line. A flag is a 64char hexstring."));
	TMemo* flagEditor = new TMemo(TRect(2, 4, 2+66, 4+10), NULL, NULL,
			NULL, 2048);
	box->insert(flagEditor);
	box->options |= ofCentered;
	
	if (execView(box) != cmCancel)
	{
		buf = new char[20480];
		// flagEditor->buffer[flagEditor->bufSize] = 0;
		tmp = flagEditor->buffer;
		memmove(tmp + flagEditor->curPtr, tmp + (flagEditor->curPtr +
				flagEditor->gapLen), flagEditor->bufLen -
				flagEditor->curPtr);
		tmp[flagEditor->bufLen] = 0;
		if (reportFlag(flagEditor->buffer, buf, 20480)) {
			messageBox("Error: set the IP of the scoring bot and your team ID correctly!",
					mfOKButton | mfError);
		} else {
			for(tmp = buf; tmp != NULL; tmp = strstr(tmp + 1, "successfully reported")) n++;
			msg = new char[512];
			snprintf(msg, 512, "%d flag%s ha%s been reported successfully."
					"If that's not what you expected, check your team ID and scoring bot address.",
					n, (n == 1) ? "" : "s", (n == 1) ? "s" : "ve");
			messageBox(msg, mfOKButton | mfInformation);
			delete msg;
		}
		delete buf;
	}

	delete box;
}

static int
testScorebot()
{
    FILE* f;
    char cmd[512];
    char botip[128];
    char* nc;
    int status;
    char buf[128];
    int l = 128;

    l--;

    nc = getenv("NETCAT");
    if (nc == NULL) {
        nc = "/usr/bin/nc";
    }

    if (djbGetSetting(".ctf_scoringbot", botip, 128) < 1) {
        return 1;
    }

    if (strlen(botip) < 1) {
        return 1;
    }

    snprintf(cmd, 512, "%s %s 8080", nc, botip);

    f = popen(cmd, "r+");
    if (f == NULL) {
        perror("popen");
        exit(-1);
    }
    fprintf(f, "quit()\n");
    fflush(f);
    wait(&status);
    l = fread(buf, 1, l, f);
    pclose(f);

    if (l >= 0) {
        buf[l] = 0;
    } else {
        buf[0] = 0;
        return 1;
    }

    if (strstr(buf, "Welcome to the CTF scorebot") == NULL) {
        return 1;
    }

    return 0;
}


int
TMyApp::testScorebotReachability()
{
	int res;
	TDialog* cd = new TDialog(TRect(0, 0, 40, 7), "Checking...");
	TStaticText* st = new TStaticText(TRect(2, 2, 2+35, 2+2),
			"Trying to contact the scoring bot...");
	cd->insert(st);
	cd->options |= ofCentered;
	deskTop->insert(cd);
	res = testScorebot();
	// sleep: hack
	if (!res) {
		cd->remove(st);
		delete st;
		st = new TStaticText(TRect(2, 2, 2+35, 2+2),
				"Connection established, please wait...");
		cd->insert(st);
		sleep(5);
	}
	deskTop->remove(cd);
	return res;
}

int
TMyApp::doSetIP()
{	
	int res = 0;

	TDialog* box = new TDialog(TRect(0, 0, 70, 15), "Set Scoring bot IP");
	box->insert(new TButton(TRect(55, 10, 55+12, 10+2), "~S~ave", cmOK, bfDefault));
	box->insert(new TButton(TRect(2, 10, 2+12, 10+2), "~C~ancel", cmCancel, 0));
	box->insert(new TStaticText(TRect(2, 2, 2+65, 2+5),
			"Enter the IP of the CTF scoring bot. Ask the admins if in doubt."));
	TInputLine* flagLine = new TInputLine(TRect(2, 5, 2+66, 5+1), 128);
	djbGetSetting(".ctf_scoringbot", flagLine->data, 128);
	box->insert(flagLine);
	box->options |= ofCentered;
	
	if (execView(box) != cmCancel)
	{
		djbSetSetting(".ctf_scoringbot", flagLine->data);
		if (testScorebotReachability()) {
			messageBox("The scorebot could not be reached at this IP", mfOKButton | mfError);
			res = 1;
		}
	} else {
		res = 2;
	}

	delete box;

	return res;
}

static int
getTeams(char* buf, int l)
{
	FILE* f;
	char cmd[512];
	char botip[128];
	char* nc;
	int status;

    l--;

	nc = getenv("NETCAT");
	if (nc == NULL) {
		nc = "/usr/bin/nc";
	}

	if (djbGetSetting(".ctf_scoringbot", botip, 128) < 1) {
		return 1;	
	}

	snprintf(cmd, 512, "%s %s 8080", nc, botip);
	// printf("%s", cmd);

	f = popen(cmd, "r+");
	if (f == NULL) {
		perror("popen");
		exit(-1);
	}
	fprintf(f, "lt()\nquit()\n");
	fflush(f);
	wait(&status);
	l = fread(buf, 1, l, f);
	pclose(f);

	if (l >= 0) {
		buf[l] = 0;
	} else {
		buf[0] = 0;
		return 1;
	}

	return 0;
}

void flagAction(char* cmd, char* buf, int l)
{
    FILE* f;

    l--;
    f = popen(cmd, "r+");
    fread(buf, 1, l, f);
    if (l > 0) {
        buf[l] = 0;
    } else {
        *buf = 0;
    }
}

int TMyApp::doSetHostIP()
{
    char *cmdline;
    char *resbuf;
	TDialog* box = new TDialog(TRect(0, 0, 70, 13), "Set IP and stuff");
	box->insert(new TButton(TRect(57, 10, 57+10, 10+2), "~T~ry", cmOK, bfDefault));
	box->insert(new TButton(TRect(2, 10, 2+12, 10+2), "~C~ancel", cmCancel, 0));
	box->insert(new TStaticText(TRect(2, 2, 2+65, 2+2),
			"Enter the IP, the netmask (255.255.0.0)\nand the defaultroute."));

	TInputLine* theIP = new TInputLine(TRect(18, 4, 40, 5), 256);
	box->insert(theIP);
	TInputLine* theMask = new TInputLine(TRect(18, 5, 40, 6), 16);
	box->insert(theMask);
	TInputLine* theRoute = new TInputLine(TRect(18, 6, 40, 7), 65);
	box->insert(theRoute);
    box->insert(new TLabel(TRect(2, 4, 18, 5), "~I~P:", theIP));
    box->insert(new TLabel(TRect(2, 5, 18, 6), "~N~etmask:", theMask));
    box->insert(new TLabel(TRect(2, 6, 18, 7), "~D~efaultroute:", theRoute));

    theIP->select();

	box->options |= ofCentered;
	if (execView(box) != cmCancel)
	{
        cmdline = new char[8192];
        if (!cmdline) {
            messageBox("Out of memory", mfOKButton | mfError);
            return 0;
        }
        suspend();
        fputs("Doing ifconfig:\n", stderr);
        snprintf(cmdline, 8192, "ifconfig le0 %s netmask %s",
                theIP->data, theMask->data);
        fputs(cmdline, stderr);
        fputs("\n", stderr);
        fflush(stderr);
        system(cmdline);
        fputs("\n\nDoing route:", stderr);
        fputs("\n", stderr);
        snprintf(cmdline, 8192, "route add default %s",
                theRoute->data);
        fputs(cmdline, stderr);
        fputs("\n", stderr);
        fflush(stderr);
        system(cmdline);
        fputs("Press ENTER to continue", stderr);
        fputs("\n", stderr);
        fflush(stderr);
        getchar();
        resume();
        delete cmdline;
	}

	delete box;

	return 0;
}




int TMyApp::doRetrieveFlag()
{
    char *cmdline;
    char *resbuf;
	TDialog* box = new TDialog(TRect(0, 0, 70, 13), "Retrieve flag");
	box->insert(new TButton(TRect(57, 10, 57+10, 10+2), "~S~tore", cmOK, bfDefault));
	box->insert(new TButton(TRect(2, 10, 2+12, 10+2), "~C~ancel", cmCancel, 0));
	box->insert(new TStaticText(TRect(2, 2, 2+65, 2+1),
			"Enter the IP, and the flag data."));

	TInputLine* flagIP = new TInputLine(TRect(12, 4, 40, 5), 256);
	box->insert(flagIP);
	TInputLine* flagID = new TInputLine(TRect(12, 5, 30, 6), 16);
	box->insert(flagID);
	TInputLine* flagCNT = new TInputLine(TRect(18, 6, 60, 7), 65);
	box->insert(flagCNT);
    box->insert(new TLabel(TRect(2, 4, 12, 5), "~I~P/Host:", flagIP));
    box->insert(new TLabel(TRect(2, 5, 12, 6), "FlagI~D~:", flagID));
    box->insert(new TLabel(TRect(2, 6, 18, 7), "Expected ~F~lag:", flagCNT));

    flagIP->select();

	box->options |= ofCentered;
	if (execView(box) != cmCancel)
	{
        cmdline = new char[8192];
        if (!cmdline) {
            messageBox("Out of memory", mfOKButton | mfError);
            return 0;
        }
        resbuf = new char[8192];
        if (!cmdline) {
            messageBox("Out of memory", mfOKButton | mfError);
            delete cmdline;
            return 0;
        }
        snprintf(cmdline, 8192, "/usr/local/bin/message.py retrieve %s %s %s",
                flagIP->data, flagID->data, flagCNT->data);
	    TDialog* waitBox = new TDialog(TRect(0, 0, 70, 13), "Retrieve flag");
        waitBox->insert(new TStaticText(TRect(4, 4, 60, 8), "Please wait"));
	    waitBox->options |= ofCentered;
        deskTop->insert(waitBox);
        flagAction(cmdline, resbuf, 8192);
        deskTop->remove(waitBox);
        delete waitBox;
        messageBox(resbuf, mfOKButton | mfInformation);
        delete resbuf;
        delete cmdline;
	}

	delete box;

	return 0;
}




int TMyApp::doStoreFlag()
{
    char *cmdline;
    char *resbuf;
	TDialog* box = new TDialog(TRect(0, 0, 70, 13), "Store flag");
	box->insert(new TButton(TRect(57, 10, 57+10, 10+2), "~S~tore", cmOK, bfDefault));
	box->insert(new TButton(TRect(2, 10, 2+12, 10+2), "~C~ancel", cmCancel, 0));
	box->insert(new TStaticText(TRect(2, 2, 2+65, 2+1),
			"Enter the IP, and the flag data."));

	TInputLine* flagIP = new TInputLine(TRect(12, 4, 40, 5), 256);
	box->insert(flagIP);
	TInputLine* flagID = new TInputLine(TRect(12, 5, 30, 6), 16);
	box->insert(flagID);
	TInputLine* flagCNT = new TInputLine(TRect(12, 6, 60, 7), 65);
	box->insert(flagCNT);
    box->insert(new TLabel(TRect(2, 4, 12, 5), "~I~P/Host:", flagIP));
    box->insert(new TLabel(TRect(2, 5, 12, 6), "FlagI~D~:", flagID));
    box->insert(new TLabel(TRect(2, 6, 12, 7), "~F~lag:", flagCNT));

    flagIP->select();

	box->options |= ofCentered;
	if (execView(box) != cmCancel)
	{
        cmdline = new char[8192];
        if (!cmdline) {
            messageBox("Out of memory", mfOKButton | mfError);
            return 0;
        }
        resbuf = new char[8192];
        if (!cmdline) {
            messageBox("Out of memory", mfOKButton | mfError);
            delete cmdline;
            return 0;
        }
        snprintf(cmdline, 8192, "/usr/local/bin/message.py store %s %s %s",
                flagIP->data, flagID->data, flagCNT->data);
	    TDialog* waitBox = new TDialog(TRect(0, 0, 70, 13), "Store flag");
        waitBox->insert(new TStaticText(TRect(4, 4, 60, 8), "Please wait"));
	    waitBox->options |= ofCentered;
        deskTop->insert(waitBox);
        flagAction(cmdline, resbuf, 8192);
        deskTop->remove(waitBox);
        delete waitBox;
        messageBox(resbuf, mfOKButton | mfInformation);
        delete resbuf;
        delete cmdline;
	}

	delete box;

	return 0;
}



int TMyApp::doSetTeamNumber()
{
	char *teams, *t, *tmp;
	int numteams = 0;
	int res = 1;
	teams = new char[4096];
	getTeams(teams, 4096);
	if (strstr(teams, "Welcome to the CTF scorebot") == NULL) {
		printf("%s\n", teams);
		messageBox("Unable to contact the scoring bot. Please set the scoring bot IP/host first.",
				mfOKButton | mfError);
		return 1;
	}
	t = strstr(teams, "ID");
	if (t == NULL) {
		t = "";
	}
	tmp = t;
	while(*tmp) {
		if (*tmp == '\t') {
			*tmp = ' ';
		}
		else if (*tmp == '\n') {
			++numteams;
		}
		++tmp;
	}
	numteams -= 4;

	TDialog* box = new TDialog(TRect(0, 0, 70, 13 + numteams), "Set your teamnumber");
	box->insert(new TButton(TRect(57, 10+numteams, 57+10, 10+2+numteams), "~S~ave", cmOK, bfDefault));
	box->insert(new TButton(TRect(2, 10+numteams, 2+12, 10+2+numteams), "~C~ancel", cmCancel, 0));
	box->insert(new TStaticText(TRect(2, 2, 2+65, 2+1),
			"Enter your teamnumber. Ask the admins if in doubt."));
	TMemo* memo = new TMemo(TRect(2, 3, 2+65, 3+numteams + 1),
			NULL, NULL, NULL, 8192);
	memo->state |= sfDisabled;
	box->insert(memo);
	memo->insertText(t, strlen(t), False);
	TInputLine* flagLine = new TInputLine(TRect(2, 7 + numteams, 2+10, 7+numteams+1), 3);
	djbGetSetting(".ctf_teamid", flagLine->data, 3);
	box->insert(flagLine);
	box->options |= ofCentered;
	if (execView(box) != cmCancel)
	{
		djbSetSetting(".ctf_teamid", flagLine->data);
		res = 0;
	}

	delete teams;
	delete box;

	return res;
}

void TMyApp::handleEvent(TEvent &event)
{
	TApplication::handleEvent(event);
	if (event.what == evCommand)
	{
		switch (event.message.command)
		{
		case cmAbout:
			aboutBox();
			break;
		case cmCascade:
			deskTop->cascade(deskTop->getExtent());
			break;
		case cmTile:
			deskTop->tile(deskTop->getExtent());
			break;
		case cmReportFlag:
			doReportFlag();
			break;
		case cmReportAdvisory:
			doReportAdv();
			break;
		case cmSetIP:
			doSetIP();
			break;
		case cmSetTeamNumber:
			doSetTeamNumber();
			break;
		case cmStoreFlag:
            doStoreFlag();
			break;
		case cmRetrieveFlag:
            doRetrieveFlag();
			break;
		case cmAFR:
			messageBox("Contact the scoring bot directly. Telnet to port 8080. Should be self-explanatory.",
					mfOKButton | mfInformation);
			break;
		default:
			return;
		}
		clearEvent(event);
	}
}

void TMyApp::idle()
{
	TApplication::idle();
}

TMenuBar* TMyApp::initMenuBar(TRect r)
{
    TSubMenu& sub1 = *new TSubMenu("~\360~", 0, hcNoContext) +
			*new TMenuItem("~A~bout...", cmAbout, kbNoKey, hcNoContext);

    TSubMenu& sub2 = *new TSubMenu("~F~ile", 0) +
			*new TMenuItem("E~x~it", cmQuit, kbAltX, hcNoContext, "Alt-X");

    TSubMenu& sub6 = *new TSubMenu("F~l~ags", 0) +
			*new TMenuItem("~S~tore flag...", cmStoreFlag, kbNoKey, hcNoContext) +
			*new TMenuItem("~R~etrieve flag...", cmRetrieveFlag, kbNoKey, hcNoContext);

    TSubMenu& sub3 = *new TSubMenu("~A~ction", 0) +
			*new TMenuItem("Report ~f~lags...", cmReportFlag, kbNoKey, hcNoContext) +
			*new TMenuItem("Report ~a~dvisory...", cmReportAdvisory, kbNoKey, hcNoContext);

    TSubMenu& sub4 = *new TSubMenu("~S~ettings", 0) +
			*new TMenuItem("Set ~s~corebot IP...", cmSetIP, kbNoKey, hcNoContext) +
			*new TMenuItem("Set ~t~eam number...", cmSetTeamNumber, kbNoKey, hcNoContext);

    TSubMenu& sub5 = *new TSubMenu("~H~elp", 0) +
			*new TMenuItem("~A~utomate flag reporting...", cmAFR, kbNoKey, hcNoContext);

    r.b.y =  r.a.y + 1;
    return new TMenuBar(r, sub1 + sub2 + sub6 + sub3 + sub4 + sub5);
}

TStatusLine* TMyApp::initStatusLine(TRect r)
{
    r.a.y = r.b.y - 1;
    return new TStatusLine(r,
	*new TStatusDef(0, 50) +
	*new TStatusItem(0, kbAltX, cmQuit) +
	*new TStatusItem("~F10~ Enter menu", kbF10, cmMenu) +
	*new TStatusItem(0, kbF5, cmZoom) +
	*new TStatusItem(0, kbCtrlF5, cmResize));
}

extern char* welcomemsg;
int main(int argc, char** argv)
{
	int term;
	char tname[1024];

	argc--; argv++;
	if(argc) {
		snprintf(tname, 1024, "/dev/%s", *argv);
		term = open(tname, O_RDWR);
		if (term == -1) {
			fprintf(stderr, "error: couldn't open %s", tname);
			return 1;
		}
        int fres, status;
        fres = fork();
        if (fres == -1) {
            fprintf(stderr, "Error: couldn't fork!\n");
            return 1;
        }
        if (fres) {
            while(wait(&status) != fres);
            system("/sbin/shutdown -p now");
            return status;
        }
		login_tty(term);
	} else {
        fprintf(stderr, "Must specify a virtual terminal as first parameter\n");
        return 1;
    }
	
	TDeskTop::defaultBkgrnd = '#';
	TMyApp a;
	TDialog* dlg = new TDialog(TRect(0, 0, 75, 20), "Welcome to the DA-Op3n testimage");
	dlg->options |= ofCentered;
	TMemo* msg = new TMemo(TRect(1, 1, 74, 15), NULL, NULL, NULL, 1024);
	msg->insertText(welcomemsg, strlen(welcomemsg) - 1, False);
	msg->state |= sfDisabled;
	dlg->insert(msg);
	dlg->insert(new TButton(TRect(32, 16, 32+10, 18), "~O~K", cmOK, bfDefault));
	a.executeDialog(dlg);
    a.doSetHostIP();
	a.run();
	return 0;
}
