#!/usr/bin/env python2.5

from sys import exit, argv, stderr
import sys
from os import popen
from time import time
from random import seed, random
import md5
import re
import socket

shit = {'parse': "Problem parsing ping result -- shouldn't happen!",
        'send' : "Problem sending packets -- no single ICMP send succeeded",
        'recv' : "Didn't receive a single ICMP response",
        'ilrp' : "Received illegal / wrong response",
        'nickreg' : "Unable to register nickname",
        'chanreg' : "Unable to register channel",
        'join' : "Unable to join channel / not alone in channel",
        'flag' : "Service delivered no flag / a wrong flag",
        'secret' : "Failed to set channel flag +s",
        'deop' : "Unable to deop myself",
        'op' : "Chanserv doesn't op me",
        'allok' : "Everything is fine",
        'identify' : "Services Identify failed",
        'nicknotregged' : "Previously registered nick was not registered anymore"}

class LineReader:
    buf = ''
    def __init__(self, socket):
        self.s = socket
    def readline(self):
        while True:
            pos = self.buf.find("\n")
            if pos != -1:
                line, self.buf = self.buf[:pos], self.buf[pos + 1:]
                return line
            frag = self.s.recv(8192)
            if frag == None: raise EOFException()
            if len(frag) == 0: raise EOFException()
            frag = frag.replace("\r", "")
            if len(frag) == 0:
                buf, self.buf = self.buf, ''
                return buf # EOF
            self.buf = self.buf + frag
    def readlineparams(self):
        line = self.readline()
        # print "RECV: %s" % repr(line)
        line = line.split(" ", 1)
        try: line = line[1]
        except: line = line[0]
        parts = line.split(":", 1)
        sparts = parts[0].strip().split(" ")
        del parts[0]
        return sparts + parts


def die(reason, exitcode = 5):
    try: msg = shit[reason]
    except: msg = "unknown reason"
    stderr.write(msg)
    exit(exitcode)

def expect(lr, row, what, err = 'ilrp'):
    try:
        while True:
            line = lr.readlineparams()
            if len(line) >= (row + 1):
                if line[row] == what:
                    print '>> found string %s in row %d <<' % (repr(what), row)
                    return
    except:
        print "Didn't catch string %s in row %d" % (what, row)
        die(err)

def store((ip, flagid, flag)):
    flagid = md5.md5(flag).hexdigest()[0:16].encode('base64').replace("\n", "").replace("=", "").lower()
    print flagid
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try: s.connect((ip, 6667))
    except:
        sys.stderr.write("Connection timed out / refused")
        sys.stderr.flush()
        exit(1)
    usr = flagid[0:10]
    s.settimeout(10.0)
    lr = LineReader(s)
    s.sendall("user %s %s %s %s\nquit\nnick %s\n" % (usr, usr, usr, usr, usr))
    expect(lr, 0, 'MODE')
    s.sendall("privmsg nickserv :register %s\n" % md5.md5(usr).hexdigest()[0:8])
    expect(lr, 2, '+e', 'nickreg')
    s.sendall("join #%s\n" % flagid)
    expect(lr, 4, '@%s' % usr, 'join')
    s.sendall("mode #%s +s\n" % flagid)
    expect(lr, 2, '+s', 'secret')
    s.sendall("privmsg chanserv :register #%s %s\n" % (flagid, flag))
    expect(lr, 2, 'Channel registered under your nick.', 'chanreg')
    if random() < 0.25:
        s.sendall("kick #%s %s :\n" % (flagid, usr))
    else:
        s.sendall("mode #%s +i-o %s\n" % (flagid, usr))
        expect(lr, 3, usr, 'deop')
    s.sendall("quit\n")
    print "===== SUCCESSFULLY STORED FLAG ====="
    die('allok', 0)

def retrieve((ip, flagid, flag)):
    flagid = md5.md5(flag).hexdigest()[0:16].encode('base64').replace("\n", "").replace("=", "").lower()
    seed(time())
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try: s.connect((ip, 6667))
    except:
        sys.stderr.write("Connection timed out / refused")
        sys.stderr.flush()
        exit(1)
    usr = flagid[0:10]
    s.settimeout(10.0)
    lr = LineReader(s)
    print 'sending user, quit, nick'
    s.sendall("user %s %s %s %s\nquit\nnick %s\n" % (usr, usr, usr, usr, usr))
    print 'expecting this nick, %s, to be registered.' % repr(usr)
    expect(lr, 2, 'This nick is registered. If it is your nick,', 'nicknotregged')
    if random() < 0.5:
        # sometimes, if we feel like it, send a wrong password first,
        # to check if the auth routine works.
        print 'identifying with services with a wrong password'
        s.sendall("privmsg nickserv :identify %s\n" % (md5.md5(usr + 'foo').hexdigest()[0:8]))
        expect(lr, 2, 'Error: Password incorrect', 'identify')
    identified = False
    # sometimes, we do not identify yet. In that case, chanserv should deop us.
    if random() < 0.5:
        print 'identifying with services'
        identified = True
        s.sendall("privmsg nickserv :identify %s\n" % (md5.md5(usr).hexdigest()[0:8]))
        expect(lr, 2, '+e', 'identify')
        # if we are identified, let's occasionally use ChanServ to forcejoin us
        # onto our channel
        if random() < 0.5:
            print 'joining channel with JOIN'
            s.sendall("join #%s\n" % flagid)
        else:
            print 'using chanserv aop FORCEJOIN'
            s.sendall("privmsg chanserv :join #%s\n" % flagid)
    else:
        print 'joining channel with JOIN while unauthed'
        s.sendall("join #%s\n" % flagid)
        expect(lr, 3, usr, 'deop')
    print 'waiting for the flag'    
    expect(lr, 2, flag, 'flag')
    # we got the flag, now let's test if the op function of chanserv works.
    if identified:
        # we're identified, so we're opped. let's deop ourselves first.
        print 'deoping ourserlves'
        s.sendall("mode #%s -o %s\n" % (flagid, usr))
        expect(lr, 3, usr, 'deop')
    else:
        # we're not identified; do that now.
        print 'late-identifying with nickserv'
        s.sendall("privmsg nickserv :identify %s\n" % (md5.md5(usr).hexdigest()[0:8]))
        expect(lr, 2, '+e', 'identify')
    # have us opped
    print 'having chanserv reop us'
    s.sendall("privmsg chanserv :op #%s\n" % flagid)
    expect(lr, 2, '+o', 'op')
    s.sendall("quit\n")
    print "===== SUCCESSFULLY RETRIEVED FLAG ====="
    die('allok', 0)

def test((ip)):
    stderr.write("Testing is integrated in store/retrieve functionality.")
    exit(0) # service is fully functional
    
try:
    {   'store': store,
        'retrieve': retrieve,
        'test': test
    }[argv[1]](tuple(argv[2:]))
except SystemExit, e: exit(e.code)
except Exception, e:
    stderr.write("ERROR! ")
    stderr.write(str(e))
    print "Error: " + str(e)
    print "Usage: %s store|retrieve IP FLAGID FLAG" % argv[0]
    print "       %s test IP" % argv[0]
finally: stderr.flush()
