#!/usr/bin/php
<?php

function run_command($ip, $port, $cmd) {
	$fp = @fsockopen("$ip", $port, $errno, $errstr, 30);
	if(!$fp) {
		echo "Can't connect\n";
		exit(1);
	}

	if(!fwrite($fp, $cmd)) {
		echo "Can't write to host";
		exit(1);
	}

	$contents = "";
	while (!feof($fp)) {
		$contents .= fgets($fp);
	}
	fclose($fp);
	return $contents;
}

function get_lines($content, $needle) {
	$out = array();
	$parts = explode("\n", $content);
	while(list($key, $line) = each($parts)) {
		if(strpos($line, $needle) !== false) {
			$out[] = $line;
		}
	}
	return $out;
}

function get_line($content, $needle) {
	$lines = get_lines($content, $needle);
	if(count($lines) < 1) {
		return "";
	} else {
		return $lines[0];
	}
}

function get_cookie($content) {
	$cookie = array();
	$line = get_line($content, "Set-Cookie:");
	$parts = explode(" ", $line);
	while(list($k, $val) = each($parts)) {
		$atoms = explode("=", $val, 2);
		if((count($atoms) == 2) && ($atoms[0] != "path")) {
			$cookie[$atoms[0]] = $atoms[1];
		}
	}
	return $cookie;
}

function flatten($array, $inner, $outer) {
	$out = array();
	while(list($key, $val) = each($array)) {
		$out[] = $key.$inner.$val;
	}
	return implode($outer, $out);
}

function http_request($ip, $method, $url, $param, $cookie, $file = array()) {
	$cmd = $method." ".$url;
	if((stripos($method, "GET") !== false) && (count($param) != 0)) {
		$cmd .= "?".flatten($param, "=", "&");
	}
	$cmd .= " HTTP/1.0\r\n";
	$cmd .= "Host: localhost\r\n";

	$cmd .= "User-Agent: Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.1pre) Gecko/2008062704 GranParadiso/3.0.1pre\r\n";
	$cmd .= "Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8\r\n";
	$cmd .= "Accept-Language: en-us,en;q=0.5\r\n";
	$cmd .= "Accept-Encoding: gzip,deflate\r\n";
	$cmd .= "Accept-Charset: UTF-8,*\r\n";
	$cmd .= "Keep-Alive: 300\r\n";

	$cmd .= "Connection: Close\r\n";
	if(count($cookie) != 0) {
		$cmd .= "Cookie: ".flatten($cookie, "=", " ")."\r\n";
	}
	if(stripos($method, "POST") !== false) {
		if(count($file) == 0) {
			$param = flatten($param, "=", "&");
			$cmd .= "Content-Type: application/x-www-form-urlencoded\r\n";
			$cmd .= "Content-Length: ".strlen($param)."\r\n\r\n";
			$cmd .= $param."\r\n";
		} else {
			$boundary = "----------".random_string(20);

			$param  = "--".$boundary."\r\n";
			$param .= "Content-Disposition: form-data; name=\"tid\"\r\n\r\n";
			$param .= $file["tid"]."\r\n";
			$param .= "--".$boundary."\r\n";
			$param .= "Content-Disposition: form-data; name=\"file\"; filename=\"{$file["name"]}\"\r\n";
			$param .= "Content-Type: text/x-java\r\n\r\n";
			$param .= $file["data"];
			$param .= "\r\n";
			$param .= "--".$boundary."--\r\n";

			$cmd .= "Content-Length: ".strlen($param)."\r\n";
			$cmd .= "Content-Type: multipart/form-data; boundary=$boundary\r\n\r\n";
			$cmd .= $param;
		}
	}
	$cmd .= "\r\n";
	// echo $cmd;
	$content = str_replace("\r\n", "\n\r", $content);
	$content = run_command($ip, 80, $cmd);
	// echo $content;
	return $content;
}

function login($ip, $user, $pass) {
	$content = http_request($ip, "GET", "/webtasks/index.php", array(), array());
	$cookie = get_cookie($content);
	// echo $content;
	$line = get_line($content, "FORM");
	if(strpos($line, "index.php") === false) {
		echo "Can't find login form\n";
		return false;
	}
	$content = http_request($ip, "POST", "/webtasks/index.php", array("user" => $user, "pass" => $pass), $cookie);
	$line = get_line($content, "Logout");
	if(strpos($line, "logout.php") === false) {
		echo "Login failed\n";
		return false;
	}
	return $cookie;
}

function logout($ip, $cookie) {
	$content = http_request($ip, "GET", "/webtasks/logout.php", array(), $cookie);
	$line = get_line($content, "logged out");
	if(strpos($line, "successfull") === false) {
		echo "Logout failed\n";
		return false;
	}
	return true;
}

function createuser($ip, $user, $pass) {
	$content = http_request($ip, "GET", "/webtasks/index.php", array(), array());
	$cookie = get_cookie($content);
	$line = get_line($content, "Register");
	if(strpos($line, "register.php") === false) {
		echo "Can't find registration link\n";
		return false;
	}
	$content = http_request($ip, "GET", "/webtasks/register.php", array(), $cookie);
	$line = get_line($content, "FORM");
	if(strpos($line, "register.php") === false) {
		echo "Can't find register form\n";
		return false;
	}
	$content = http_request($ip, "POST", "/webtasks/register.php", array("user" => $user, "pass" => $pass), $cookie);
	$line = get_line($content, "Registration");
	if(strpos($line, "successfull. Please login in") === false) {
		echo "Can't register\n";
		return false;
	}
	return true;
}

function changepassword($ip, $pass, $new, $cookie) {
	$content = http_request($ip, "GET", "/webtasks/profil.php", array(), $cookie);
	$line = get_line($content, "FORM");
	if(strpos($line, "passwd.php") === false) {
		echo "Can't find form for password change\n";
		return false;
	}
	$content = http_request($ip, "POST", "/webtasks/passwd.php", array("oldpassword" => $pass, "newpassword1" => $new, "newpassword2" => $new), $cookie);
	$line = get_line($content, "succesfully");
	if(strpos($line, "Password changed succesfully") === false) {
		echo "Can't change password\n";
		return false;
	}
	return true;
}

function random_string($len) {
	$out = "";
	$chars = "0123456789ABCDEFGIJKLMNOPQRSTUVQWYZabcdefghijklmnopqrstuvwxyz_";
	for($i = 0; $i < $len; $i++) {
		$out .= $chars[rand(0, strlen($chars) - 1)];
	}
	return $out;
}

function get_account($ip, $user = "") {
	$file = "account_".$ip;
	$account = array("user" => "", "pass" => "");
	if(!is_file($file)) {
		return $account;
	}
	$parts = explode("\n", file_get_contents($file));
	while(list($lineno, $line) = each($parts)) {
		$atoms = explode(" ", $line);
		if(count($atoms) == 2) {
			if(($user == "") || ($atoms[0] == $user)) {
				$account["user"] = rtrim($atoms[0], " \r\n");
				$account["pass"] = rtrim($atoms[1], " \r\n");
			}
		}
	}
	return $account;
}

function set_account($ip, $user, $pass) {
	$file = "account_".$ip;
	if(($fp = fopen($file, "a")) === false) {
		echo "Can't open file\n";
		return false;
	}
	$account = $user." ".$pass."\n";
	$success = fwrite($fp, $account);
	if(($success === false) || ($success < strlen($account))) {
		echo "Write failed\n";
		return false;
	}
	fclose($fp);
	return true;
}

function create_account($ip) {
	$user = random_string(15);
	$pass = random_string(15);
	$success = createuser($ip, $user, $pass);
	if($success === false) {
		echo "Can't create account\n";
		return false;
	}
	return array("user" => $user, "pass" => $pass);
}

function retrieve($ip, $flagid, $flag) {
	$lines = get_lines(file_get_contents("flags_".$ip), $flagid);
	$line = $lines[count($lines) - 1]; // Use most recent
	$parts = explode(" ", $line);
	if(count($parts) < 5) {
		echo "Can't find info about $flagid\n";
		return false;
	}
	$tid = rtrim($parts[1], ", ");
	$sid = rtrim($parts[2], ", ");
	$user = rtrim($parts[3], ", ");
	$kind = rtrim($parts[4], ", ");
	$account = get_account($ip, $user);
	$cookie = login($ip, $account["user"], $account["pass"]);
	$content = http_request($ip, "GET", "/webtasks/assignment.php", array("tid" => $tid), $cookie);
	$line = get_line($content, "Solution $sid");
	if($kind == "working") {
		if(strpos($line, "testok") === false) {
			echo "Programm not recognized as working\n";
			return false;
		}
	} else { // kind = broken
		if(strpos($line, "testfailed") === false) {
			echo "Programm breakage not detected\n";
			return false;
		}
	}
	$content = http_request($ip, "GET", "/webtasks/getsource.php", array("sid" => $sid), $cookie);
	$line = explode(" ", get_line($content, " * "), 3);
	if(rtrim($line[2], " \r\n") != $flag) {
		echo "Flag missmatch\n";
		exit(5);
	}
	return true;
}

function store($ip, $flagid, $flag) {
	$tid = rand(1, 5);
	if(rand(0, 1) == 0) {
		$kind = "working";
	} else {
		$kind = "broken";
	}
	$data = file_get_contents("solutions/".$tid."_".$kind.".java");
	$data = str_replace("PLACEFLAGHERE", $flag, $data);
	$file = array("tid" => $tid, "name" => $tid."_".$kind.".java", "data" => $data);
	$account = get_account($ip);
	if(($account["user"] == "") || ($account["pass"] == "")) {
		echo "Account information missing\n";
		return false;
	}
	$cookie = login($ip, $account["user"], $account["pass"]);
	if($cookie === false) {
		echo "Can't login\n";
		return false;
	}
	$content = http_request($ip, "POST", "/webtasks/solution.php", array(), $cookie, $file);
	$line = get_line($content, "Storing");
	if(strpos($line, "successfull") === false) {
		echo "Storing solution failed\n";
		return false;
	}
	$content = http_request($ip, "GET", "/webtasks/assignment.php", array("tid" => $tid), $cookie);
	$line = get_line($content, "Solution ");
	$parts = explode(" ", $line);
	$sid = $parts[2];
	if(($fp = fopen("flags_".$ip, "a")) === false) {
		echo "Can't open state file\n";
		return false;
	}
	$success = fwrite($fp, "$flagid, $tid, $sid, {$account["user"]}, $kind \n");
	if($success === false) {
		echo "Can't write to state file\n";
		return false;
	}
	fclose($fp);
	$content = http_request($ip, "GET", "/webtasks/dotest.php", array("sid" => $sid), $cookie);
	return true;
}

function test($ip) {
	$account = get_account($ip);
	if(($account["user"] == "") || ($account["pass"] == "")) {
		$cookie = false;
	} else {
		$cookie = login($ip, $account["user"], $account["pass"]);
		if($cookie === false) {
			echo "Login failed\n";
		}
	}
	if($cookie === false) {
		$account = create_account($ip);
		if($account === false) {
			echo "Can't create account\n";
			return false;
		}
		$cookie = login($ip, $account["user"], $account["pass"]);
		if($cookie === false) {
			echo "Can't login\n";
			return false;
		}
	}
	$new = random_string(15);
	$success = changepassword($ip, $account["pass"], $new, $cookie);
	if($success === false) {
		echo "Can't change password\n";
		return false;
	}
	$success = set_account($ip, $account["user"], $new);
	if($success === false) {
		echo "Saving account data failed\n";
	}
	return true;
}


/////////////////////////////////////////////////////////////////////
// MAIN                                                            //
/////////////////////////////////////////////////////////////////////

if(isset($argv[1])) {
	$what = $argv[1];
} else {
	echo "no todo\n";
	exit(-1);
}

if(isset($argv[2])) {
	$ip = $argv[2];
} else {
	echo "no ip\n";
	exit(-1);
}

if($what == "store" || $what == "retrieve") {
	if(isset($argv[3])) {
		$flagid = $argv[3];
	} else {
		echo "no flagid\n";
		exit(-1);
	}

	if(isset($argv[4])) {
		$flag = $argv[4];
	} else {
		echo "no flag\n";
		exit(-1);
	}
	if($what == "store") {
		$success = store($ip, $flagid, $flag);
		if($success === false) {
			echo "Storing the flag failed\n";
			exit(9);
		} else {
			exit(0);
		}
	} else {
		$success = retrieve($ip, $flagid, $flag);
		if($success === false) {
			echo "Getting the flag failed\n";
			exit(9);
		} else {
			exit(0);
		}
	}
} else if($what == "test") {
	$success = test($ip);
	if($success === false) {
		echo "Test failed";
		exit(9);
	} else {
		exit(0);
	}
} else {
	echo "Unknown todo. Use store, retrieve or test";
	exit(-1);
}
?>
