#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "lua.h"
#include "lauxlib.h"
#include "lualib.h"

static void luae(lua_State *L, const char *err)
{
	lua_pushnil(L);
	lua_pushstring(L, err);
}

static int l_dxchg(lua_State *L)
{
	const char *host = luaL_checkstring(L, 1);
	const int port = luaL_checknumber(L, 2);
	const char *dts = luaL_checkstring(L, 3);

	char *buf;
	int s;
	int retval;
	struct sockaddr_in addr;
	int bufsize;

	if (strlen(dts) < 3) {
		luae(L, "No proper data supplied");
		return 2;
	}

	/* No IP address we allow is shorter than 10.1.1.1 (8 chars) */
	if (strlen(host) < 8) {
		luae(L, "No");
		return 2;
	}

	/* IP address must start with 10. or 127. */
	if (memcmp(host, "127.", 4) &
	    memcmp(host, "10.", 3)) {
		luae(L, "No, pal");
		return 2;
	}

	/* Awwritte, now a socket, please */
	s = socket(AF_INET, SOCK_STREAM, htons(0));
	if (!s) {
		luae(L, "socket");
		return 2;
	}

	/* parse IP string */
	retval = inet_aton(host, &addr.sin_addr);
	if (!retval) {
		luae(L, "no valid ip");
		return 2;
	}

	/* Set some paramz */
	addr.sin_family = AF_INET;
	addr.sin_port = htons(port);

	/* Nowwz, connect */
	retval = connect(s, (struct sockaddr*) &addr, sizeof(addr));
	if (retval) {
		luae(L, "Connect failed");
		return 2;
	}

	/* OK. Now we send data */
	retval = send(s, dts + 2, strlen(dts) - 2, MSG_NOSIGNAL);
	if (retval < 0) {
		luae(L, "Send failed");
		return 2;
	}

	/* Allocate the buffer */
	bufsize = (dts[0] << 8) + dts[1];
	buf = malloc(bufsize);
	if (!buf) {
		luae(L, "malloc failed, ftw!");
		return 2;
	}

	*buf = 0;
	retval = recv(s, buf, bufsize, MSG_WAITALL);
	if (retval > 0) buf[retval] = 0;

	lua_pushstring(L, buf);
	free(buf);
	return 1;
}

int main ()
{
	int error;
	char buff[10240];
	int fd;

	lua_State *L = lua_open();   /* opens Lua */

	luaopen_base(L);
	luaopen_string(L);
	luaopen_table(L);
	luaopen_math(L);

	lua_pushcfunction(L, l_dxchg);
	lua_setglobal(L, "dxchg");

	fd = open("footest", O_RDONLY);
	read(fd, buff, 10240);
	close(fd);
	
	error = luaL_loadbuffer(L, buff, strlen(buff), "line") ||
	lua_pcall(L, 0, 0, 0);
	if (error) {
		fprintf(stderr, "%s", lua_tostring(L, -1));
		lua_pop(L, 1);  /* pop error message from the stack */
	}
    
	lua_close(L);
	return 0;
}

